{
   Unit Name      : MILFlashLabel.pas
   Component Name : TMILFlashLabel
   Author         : Larry J. Rutledge ( lrutledge@jps.net )
   Created        : December 4, 1998
   Last Modified  : N/A

   Description    : This component reproduces the label control on the Delphi 3
                    installation menu. When the mouse is away from it, it looks
                    like a standard label. When the mouse moves over it, a
                    selected highlight color appears around the label and when
                    the label is clicked it moves like a button.

   Instructions   : To use the component, drop it onto a form and set the
                    normal and highlight colors. Then assign methods to the
                    OnClick to specify what action to perform when the label
                    is clicked.

   Important Note : No notes for this component

   History        : Version  Date         Author     Description
                    -------  ----------   ------     --------------------------
                    1.0.0    12/04/1998    LJR       Created initial component.


   Proposed
   Revisions      : Add an image (icon or small bitmap) to the control. Moving
                    the mouse over the image would also highlight the label.
                    Clicking on the image would also trigger the OnClick event.
}

unit MILFlashLabel;

interface

uses
  Classes, Controls, DsgnIntf, Forms, Messages, StdCtrls, Windows, Graphics;

const
  _TITLE     = 'TMILFlashLabel';
  _VERSION   = '1.0.0';
  _AUTHOR    = 'Larry J. Rutledge';
  _COPYRIGHT = ' Copyright 1998, Millennial Software';

type
  { TAboutProperty - A Property editor for the About property. Displays an    }
  {                  about window when the ellipses are clicked.              }
  TAboutProperty = class(TPropertyEditor)
  public
    function  GetValue      : String; override;
    function  GetAttributes : TPropertyAttributes; override;

    procedure Edit; override;
  end;

  TMILFlashLabel = class(TCustomControl)
  private
    FAbout : TAboutProperty;

    FCaption : TCaption;
    FHighlightColor : TColor;
    FHovering : Boolean;
    FMouseIsDown : Boolean;
    FNormalColor : TColor;

    FClick : TNotifyEvent;
    FMouseEnter : TNotifyEvent;
    FMouseExit : TNotifyEvent;

    procedure SetCaption(Value : TCaption);
    procedure SetHighlightColor(Value : TColor);
    procedure SetHovering(Value : Boolean);
    procedure SetMouseIsDown(Value : Boolean);
    procedure SetNormalColor(Value : TColor);
  protected
    procedure Paint; override;

    procedure TCMMFontChanged(var Msg : TMessage); message CM_FONTCHANGED;
    procedure TCMMouseEnter(var Msg : TMessage); message CM_MOUSEENTER;
    procedure TCMMouseLeave(var Msg : TMessage); message CM_MOUSELEAVE;

    procedure MouseDown(Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton;
                      Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor  Destroy; override;
  published
    property About          : TAboutProperty  read FAbout
                                             write FAbout;
    property Caption        : TCaption        read FCaption
                                             write SetCaption;
    property HighlightColor : TColor          read FHighlightColor
                                             write SetHighlightColor;
    property Hover          : Boolean         read FHovering
                                             write SetHovering;
    property MouseIsDown    : Boolean         read FMouseIsDown
                                             write SetMouseIsDown;
    property NormalColor    : TColor          read FNormalColor
                                             write SetNormalColor;

    property Font;

    property OnClick : TNotifyEvent read FClick write FClick;
    property OnMouseEnter : TNotifyEvent read FMouseEnter write FMouseEnter;
    property OnMouseExit : TNotifyEvent read FMouseExit write FMouseExit;
  end;

procedure Register;

implementation

uses
  MILAbout;

{---------------------------------------------------------}
{ TAboutProperty - About Form Property Editor methods     }
{---------------------------------------------------------}

(******************************************************************************)
(* GetValue      -  Called when Delphi wants to display the value of the      *)
(*                  property in the Object Inspector.                         *)
(******************************************************************************)
function  TAboutProperty.GetValue : String;
begin
  Result := '(about)';
end;

(******************************************************************************)
(* GetAttributes -  Called when Delphi wants to know what attributes to apply *)
(*                  to the property.                                          *)
(******************************************************************************)
function  TAboutProperty.GetAttributes : TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paReadOnly];
end;

(******************************************************************************)
(* Edit          -  Called when the user clicks the ellipses in the value     *)
(*                  field.                                                    *)
(******************************************************************************)
procedure TAboutProperty.Edit;
begin
  with TfrmMILAbout.Create(Application) do
  try
    SetLabels(_TITLE, _VERSION, _AUTHOR, _COPYRIGHT);
    ShowModal;
  finally
    Free;
  end;
end;

constructor TMILFlashLabel.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  Width    := 45;
  Height   := 18;

  FNormalColor    := clBlack;
  FHighlightColor := clYellow;

  FCaption  := 'Caption';
  FHovering := False;
end;

destructor TMILFlashLabel.Destroy;
begin
  inherited Destroy;
end;

procedure TMILFlashLabel.Paint;
var
  R : TRect;

begin
  R := GetClientRect;

  Canvas.Font.Assign(Font);
  Canvas.Brush.Style := bsClear;
  Canvas.FillRect(R);

  if FHovering then
  begin
    // Draw highlighted text
    with Canvas do
    begin
      Font.Color := FHighlightColor;

      if FMouseIsDown then
      begin
        TextOut(R.Left + 3, R.Top + 3, FCaption);
        TextOut(R.Left + 5, R.Top + 5, FCaption);
      end else
      begin
        TextOut(R.Left + 1, R.Top + 1, FCaption);
        TextOut(R.Left + 3, R.Top + 3, FCaption);
      end;
    end;
  end;

  // draw text
  Canvas.Font.Color := FNormalColor;

  if FMouseIsDown then
    Canvas.TextOut(R.Left + 4, R.Top + 4, FCaption)
  else
    Canvas.TextOut(R.Left + 2, R.Top + 2, FCaption);
end;

procedure TMILFlashLabel.SetCaption(Value : TCaption);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    Width    := Canvas.TextWidth(FCaption) + 6;
    Height   := Canvas.TextHeight(FCaption) + 6;

    Invalidate;
  end;
end;

procedure TMILFlashLabel.SetHovering(Value : Boolean);
begin
  if FHovering <> Value then
  begin
    FHovering := Value;
    Invalidate;
  end;
end;

procedure TMILFlashLabel.SetMouseIsDown(Value : Boolean);
begin
  if FMouseIsDown <> Value then
  begin
    FMouseIsDown := Value;
    Invalidate;
  end;
end;

procedure TMILFlashLabel.SetNormalColor(Value : TColor);
begin
  if FNormalColor <> Value then
  begin
    FNormalColor := Value;
    Invalidate;
  end;
end;

procedure TMILFlashLabel.SetHighlightColor(Value : TColor);
begin
  if FHighlightColor <> Value then
  begin
    FHighlightColor := Value;
    Invalidate;
  end;
end;

procedure TMILFlashLabel.TCMMFontChanged(var Msg : TMessage);
begin
  Canvas.Font.Assign(Font);
  
  Width    := Canvas.TextWidth(FCaption) + 6;
  Height   := Canvas.TextHeight(FCaption) + 6;

  Invalidate;
end;

procedure TMILFlashLabel.TCMMouseEnter(var Msg : TMessage);
begin
  inherited;

  FHovering := True;
  Invalidate;

  if Assigned(FMouseEnter) then
    FMouseEnter(Self);
end;

procedure TMILFlashLabel.TCMMouseLeave(var Msg : TMessage);
begin
  inherited;

  FHovering := False;
  Invalidate;

  if Assigned(FMouseEnter) then
    FMouseExit(Self);
end;

procedure TMILFlashLabel.MouseDown(Button: TMouseButton;
                                   Shift: TShiftState; X, Y: Integer);
begin
  FMouseIsDown := True;
  Invalidate;

  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TMILFlashLabel.MouseUp(Button: TMouseButton;
                                 Shift: TShiftState; X, Y: Integer);
begin
  FMouseIsDown := False;
  Invalidate;

  if (FHovering) and (Assigned(FClick)) then
    FClick(Self);

  inherited MouseUp(Button, Shift, X, Y);
end;

procedure Register;
begin
  // Register Property Editors
  RegisterPropertyEditor(TypeInfo(TAboutProperty), TMILFlashLabel, 'About',
                         TAboutProperty);

  RegisterComponents('Millennial', [TMILFlashLabel]);
end;

end.
